/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { EMPTY_STRING } from '@alfa-client/tech-shared';
import { Antragsteller } from '@alfa-client/vorgang-shared';
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatRipple } from '@angular/material/core';
import { MatIcon } from '@angular/material/icon';
import { MatList, MatListItem } from '@angular/material/list';
import { getElementFromFixture } from 'libs/test-utils/src/lib/helper';
import { createAntragsteller } from 'libs/vorgang-shared/test/vorgang';
import { VorgangDetailAntragstellerComponent } from './vorgang-detail-antragsteller.component';

registerLocaleData(localeDe, 'de');

describe('VorgangDetailAntragstellerComponent', () => {
  let component: VorgangDetailAntragstellerComponent;
  let fixture: ComponentFixture<VorgangDetailAntragstellerComponent>;

  const antragstellerName: string = '[data-test-id="antragsteller-name"]';
  const antragstellerAnsprechspartnerName: string =
    '[data-test-id="antragsteller-ansprechpartner-name"]';
  const antragstellerFirmaName: string = '[data-test-id="antragsteller-firma-name"]';
  const antragstellerEmail: string = '[data-test-id="antragsteller-email"]';
  const antragstellerTelefon: string = '[data-test-id="antragsteller-telefon"]';
  const antragstellerStrasseHausnummer: string =
    '[data-test-id="antragsteller-strasse-hausnummer"]';
  const antragstellerPlzOrt: string = '[data-test-id="antragsteller-plz-ort"]';
  const antragstellerGeburt: string = '[data-test-id="antragsteller-geburt"]';
  const antragstellerAdresse: string = '[data-test-id="antragsteller-adresse"]';
  const antragstellerList: string = '[data-test-id="antragsteller-list"]';
  const noAntragsteller: string = '[data-test-id="no-antragsteller"]';

  const antragsteller: Antragsteller = createAntragsteller();
  const antragstellerWithoutCompany = { ...antragsteller, firmaName: '' };

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [VorgangDetailAntragstellerComponent],
      imports: [MatIcon, MatList, MatListItem, MatRipple],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(VorgangDetailAntragstellerComponent);
    component = fixture.componentInstance;
    component.antragsteller = antragsteller;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('full name', () => {
    it('should be visible, if applicant is not company', () => {
      component.antragsteller = antragstellerWithoutCompany;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, antragstellerName);

      expect(element).toHaveTextContent(component.name);
    });

    it('should not be visible, if applicant is company', () => {
      component.antragsteller = antragsteller;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, antragstellerName);

      expect(element).not.toBeInstanceOf(HTMLElement);
    });

    it('should not be visible, if contact person name is empty', () => {
      component.antragsteller = {
        ...antragstellerWithoutCompany,
        anrede: '',
        vorname: '',
        nachname: '',
      };
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, antragstellerName);

      expect(element).not.toBeInstanceOf(HTMLElement);
    });

    it('should trim multiple spaces to one space', () => {
      component.antragsteller = {
        ...antragsteller,
        anrede: 'huhu',
        vorname: null,
        nachname: 'duda',
      };

      expect(component.name).toStrictEqual('huhu duda');
    });
  });

  describe('full name contact person', () => {
    it('should be visible, if applicant is company', () => {
      component.antragsteller = antragsteller;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, antragstellerAnsprechspartnerName);

      expect(element).toHaveTextContent(component.name);
    });

    it('should not be visible, if applicant is not company', () => {
      component.antragsteller = antragstellerWithoutCompany;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, antragstellerAnsprechspartnerName);

      expect(element).not.toBeInstanceOf(HTMLElement);
    });

    it('should not be visible, if name is empty', () => {
      component.antragsteller = { ...antragsteller, anrede: '', vorname: '', nachname: '' };
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, antragstellerName);

      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });

  describe('firma name', () => {
    it('should be visible', () => {
      component.antragsteller = antragsteller;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, antragstellerFirmaName);

      expect(element).toHaveTextContent(component.antragsteller.firmaName);
    });

    it('should not be visible', () => {
      component.antragsteller = antragstellerWithoutCompany;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, antragstellerFirmaName);

      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });
  describe('isFirma', () => {
    it('should return true', () => {
      component.antragsteller = antragsteller;

      const isFirma = component.isFirma;

      expect(isFirma).toBeTruthy;
    });

    it('should return false', () => {
      component.antragsteller = antragstellerWithoutCompany;

      const isFirma = component.isFirma;

      expect(isFirma).toBeFalsy;
    });
  });

  describe('email', () => {
    it('should be visible', () => {
      component.antragsteller = antragsteller;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, antragstellerEmail);

      expect(element).toHaveTextContent(antragsteller.email);
    });

    it('should not be visible', () => {
      component.antragsteller = { ...antragsteller, email: '' };
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, antragstellerEmail);

      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });

  describe('telefon number', () => {
    it('should be visible', () => {
      component.antragsteller = antragsteller;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, antragstellerTelefon);

      expect(element).toHaveTextContent(antragsteller.telefon);
    });

    it('should not be visible', () => {
      component.antragsteller = { ...antragsteller, telefon: '' };
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, antragstellerTelefon);

      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });

  describe('geburtsdata', () => {
    it('should show formatted geburtsdatum and geburtsort', () => {
      const geburtsdatum: string = '2022-01-01';
      component.antragsteller = { ...antragsteller, geburtsdatum };
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, antragstellerGeburt);

      expect(element).toHaveTextContent(`01.01.2022 in ${antragsteller.geburtsort}`);
    });

    it('should show not formatted geburtsdatum on unparsable string and geburtsort', () => {
      const geburtsdatum: string = 'quatsch';
      component.antragsteller = { ...antragsteller, geburtsdatum };
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, antragstellerGeburt);

      expect(element).toHaveTextContent(`${geburtsdatum} in ${antragsteller.geburtsort}`);
    });
  });

  describe('adresse', () => {
    beforeEach(() => {
      component.antragsteller = antragsteller;
      fixture.detectChanges();
    });

    it('should have strasse + hausnummer of Antragstellerin', () => {
      const element = getElementFromFixture(fixture, antragstellerStrasseHausnummer);

      expect(element).toHaveTextContent(component.antragstellerStrasseHausnummer);
    });

    it('should have plz + ort of Antragstellerin', () => {
      const element = getElementFromFixture(fixture, antragstellerPlzOrt);

      expect(element).toHaveTextContent(component.antragstellerPlzOrt);
    });

    it('should be visible', () => {
      const element = getElementFromFixture(fixture, antragstellerAdresse);

      expect(element).toBeInstanceOf(HTMLElement);
    });

    it('should not be visible', () => {
      component.antragsteller = { ...antragsteller, strasse: '', hausnummer: '', plz: '', ort: '' };
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, antragstellerAdresse);

      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });

  describe('antragsteller existing', () => {
    beforeEach(() => {
      component.antragsteller = antragsteller;
      fixture.detectChanges();
    });

    it('mat-list should be visible', () => {
      const element = getElementFromFixture(fixture, antragstellerList);

      expect(element).toBeInstanceOf(HTMLElement);
    });

    it('noAntragsteller should not be visible', () => {
      const element = getElementFromFixture(fixture, noAntragsteller);

      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });

  describe('antragsteller empty', () => {
    beforeEach(() => {
      component.antragsteller = createEmptyAntragsteller();
      fixture.detectChanges();
    });

    function createEmptyAntragsteller(): Antragsteller {
      return {
        anrede: EMPTY_STRING,
        geburtsdatum: EMPTY_STRING,
        geburtsname: EMPTY_STRING,
        geburtsort: EMPTY_STRING,
        nachname: EMPTY_STRING,
        vorname: EMPTY_STRING,
        firmaName: EMPTY_STRING,
        email: EMPTY_STRING,
        telefon: EMPTY_STRING,
        strasse: EMPTY_STRING,
        hausnummer: EMPTY_STRING,
        plz: EMPTY_STRING,
        ort: EMPTY_STRING,
        otherData: null,
      };
    }

    it('mat-list should not be visible', () => {
      const element = getElementFromFixture(fixture, antragstellerList);

      expect(element).not.toBeInstanceOf(HTMLElement);
    });

    it('noAntragsteller should be visible', () => {
      const element = getElementFromFixture(fixture, noAntragsteller);

      expect(element).toBeInstanceOf(HTMLElement);
    });
  });

  describe('antragsteller object is null', () => {
    beforeEach(() => {
      component.antragsteller = null;
      fixture.detectChanges();
    });

    it('mat-list should not be visible', () => {
      const element = getElementFromFixture(fixture, antragstellerList);

      expect(element).not.toBeInstanceOf(HTMLElement);
    });

    it('noAntragsteller should be visible', () => {
      const element = getElementFromFixture(fixture, noAntragsteller);

      expect(element).toBeInstanceOf(HTMLElement);
    });
  });
});
