import {
  Bescheid,
  BescheidLinkRel,
  BescheidResource,
  BescheidSendBy,
  BescheidService,
  DocumentResource,
} from '@alfa-client/bescheid-shared';
import { BinaryFileResource } from '@alfa-client/binary-file-shared';
import { tapOnCommandSuccessfullyDone } from '@alfa-client/command-shared';
import {
  AbstractFormService,
  convertToBoolean,
  EMPTY_STRING,
  formatForDatabase,
  HttpError,
  isNotEmpty,
  isNotNil,
  StateResource,
} from '@alfa-client/tech-shared';
import { VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { Injectable, OnDestroy } from '@angular/core';
import {
  FormControl,
  UntypedFormArray,
  UntypedFormBuilder,
  UntypedFormControl,
  UntypedFormGroup,
} from '@angular/forms';
import { getUrl, hasLink, Resource, ResourceUri } from '@ngxp/rest';
import { isEmpty, isNil, isUndefined } from 'lodash-es';
import {
  BehaviorSubject,
  combineLatest,
  map,
  Observable,
  startWith,
  Subject,
  Subscription,
} from 'rxjs';

@Injectable()
export class BescheidenFormService extends AbstractFormService implements OnDestroy {
  static readonly FIELD_BESCHIEDEN_AM = 'beschiedenAm';
  static readonly FIELD_BEWILLIGT = 'bewilligt';
  static readonly FIELD_BESCHEID_DOCUMENT = 'bescheidDocument';
  static readonly FIELD_ATTACHMENTS = 'attachments';
  public static readonly FIELD_SEND_BY = 'sendBy';
  static readonly FIELD_NACHRICHT_SUBJECT = 'nachrichtSubject';
  static readonly FIELD_NACHRICHT_TEXT = 'nachrichtText';

  static readonly FIELD_PATH_PREFIX = 'command.body';

  static readonly BETREFF_DEFAULT: string = 'Ihr Bescheid zum Antrag';
  static readonly NACHRICHTENTEXT_BEWILLIGT_DEFAULT: string = [
    'Sehr geehrte/r Antragsteller/in,',
    'Ihr Antrag wurde bewilligt.',
    'Mit freundlichen Grüßen',
    'Ihre Verwaltung',
  ].join('\n\n');
  static readonly NACHRICHTENTEXT_ABGELEHNT_DEFAULT: string = [
    'Sehr geehrte/r Antragsteller/in,',
    'Ihr Antrag wurde abgelehnt.',
    'Mit freundlichen Grüßen',
    'Ihre Verwaltung',
  ].join('\n\n');

  private readonly bescheidChanges$: BehaviorSubject<Bescheid>;
  private readonly fileDelete$: Subject<BinaryFileResource>;
  private readonly showMissingBescheidDocumentError$: BehaviorSubject<boolean> =
    new BehaviorSubject<boolean>(false);

  private readonly activeStep$: BehaviorSubject<number> = new BehaviorSubject(1);
  readonly sendByManual: BehaviorSubject<boolean> = new BehaviorSubject(false);

  vorgangWithEingangResource: VorgangWithEingangResource;

  private formControlSubscriptions: Subscription;
  private formChangesSubscription: Subscription;

  constructor(
    formBuilder: UntypedFormBuilder,
    private readonly bescheidService: BescheidService,
  ) {
    super(formBuilder);

    this.bescheidChanges$ = new BehaviorSubject<Bescheid>(this.getFormValue());
    this.fileDelete$ = new Subject<BinaryFileResource>();
    this.init();
  }

  init(): void {
    this.formControlSubscriptions = this.subscribeToSendBy();
    this.initializeFormChanges();
  }

  subscribeToSendBy(): Subscription {
    return this.form.controls[BescheidenFormService.FIELD_SEND_BY].valueChanges.subscribe(
      (sendBy: BescheidSendBy) => this.updateSendByManual(sendBy),
    );
  }

  updateSendByManual(sendBy: BescheidSendBy): void {
    this.sendByManual.next(sendBy === BescheidSendBy.MANUAL);
  }

  initializeFormChanges(): void {
    this.formChangesSubscription = this.form.valueChanges
      .pipe(
        startWith(this.getFormValue()),
        map((value) => ({
          ...value,
          bewilligt: convertToBoolean(value.bewilligt),
        })),
      )
      .subscribe((bescheid) => this.bescheidChanges$.next(bescheid));
  }

  ngOnDestroy(): void {
    this.unsubscribe();
  }

  private unsubscribe(): void {
    if (isNotNil(this.formChangesSubscription) && !this.formChangesSubscription.closed) {
      this.formChangesSubscription.unsubscribe();
    }
    if (isNotNil(this.formControlSubscriptions)) this.formControlSubscriptions.unsubscribe();
  }

  public patchValues(bescheid: BescheidResource): void {
    if (isNil(bescheid)) {
      return;
    }
    const bescheidDocumentUri: ResourceUri = this.getBescheidDocumentUri(bescheid);
    this.bescheidService.setDocumentUri(bescheidDocumentUri);
    this.patch({
      [BescheidenFormService.FIELD_BESCHIEDEN_AM]: bescheid.beschiedenAm,
      [BescheidenFormService.FIELD_BEWILLIGT]: String(bescheid.bewilligt),
      [BescheidenFormService.FIELD_BESCHEID_DOCUMENT]: bescheidDocumentUri,
      [BescheidenFormService.FIELD_SEND_BY]:
        isUndefined(bescheid.sendBy) ? BescheidSendBy.NACHRICHT : bescheid.sendBy,
      [BescheidenFormService.FIELD_NACHRICHT_SUBJECT]: bescheid.nachrichtSubject,
      [BescheidenFormService.FIELD_NACHRICHT_TEXT]: bescheid.nachrichtText,
    });
    bescheid.attachments.forEach((attachmentLink) =>
      (this.form.controls[BescheidenFormService.FIELD_ATTACHMENTS] as UntypedFormArray).push(
        new UntypedFormControl(attachmentLink),
      ),
    );
  }

  private getBescheidDocumentUri(bescheid: BescheidResource): ResourceUri {
    if (hasLink(bescheid, BescheidLinkRel.BESCHEID_DOCUMENT)) {
      return getUrl(bescheid, BescheidLinkRel.BESCHEID_DOCUMENT);
    }
    return null;
  }

  protected initForm(): UntypedFormGroup {
    return this.formBuilder.group({
      [BescheidenFormService.FIELD_BESCHIEDEN_AM]: new UntypedFormControl(new Date(Date.now())),
      [BescheidenFormService.FIELD_BEWILLIGT]: new UntypedFormControl('true'),
      [BescheidenFormService.FIELD_SEND_BY]: new UntypedFormControl(BescheidSendBy.NACHRICHT),
      [BescheidenFormService.FIELD_BESCHEID_DOCUMENT]: new UntypedFormControl(null),
      [BescheidenFormService.FIELD_ATTACHMENTS]: new UntypedFormArray([]),
      [BescheidenFormService.FIELD_NACHRICHT_SUBJECT]: new UntypedFormControl(''),
      [BescheidenFormService.FIELD_NACHRICHT_TEXT]: new UntypedFormControl(''),
    });
  }

  protected getPathPrefix(): string {
    return BescheidenFormService.FIELD_PATH_PREFIX;
  }

  protected doSubmit(): Observable<StateResource<Resource | HttpError>> {
    if (this.isPatch() || this.bescheidService.existsBescheidDraft()) {
      return this.bescheidService.updateBescheid(this.getValue());
    } else {
      return this.bescheidService.createBescheid(this.vorgangWithEingangResource, this.getValue());
    }
  }

  public submitDraft(): Observable<StateResource<Resource | HttpError>> {
    return this.submit().pipe(
      tapOnCommandSuccessfullyDone(() => this.bescheidService.reloadCurrentVorgang()),
    );
  }

  public getValue(): Bescheid {
    const value: any = this.getFormValue();
    return {
      ...value,
      beschiedenAm: formatForDatabase(value.beschiedenAm),
      bewilligt: convertToBoolean(value.bewilligt),
      attachments: value.attachments,
    };
  }

  public getBescheidChanges(): Observable<Bescheid> {
    return this.bescheidChanges$.asObservable();
  }

  public setVorgangWithEingangResource(
    vorgangWithEingangResource: VorgangWithEingangResource,
  ): void {
    this.vorgangWithEingangResource = vorgangWithEingangResource;
  }

  public getVorgangWithEingangResource(): VorgangWithEingangResource {
    return this.vorgangWithEingangResource;
  }

  public deleteFile(binaryFileResource: BinaryFileResource): void {
    this.fileDelete$.next(binaryFileResource);
  }

  public getFileDelete(): Observable<BinaryFileResource> {
    return this.fileDelete$.asObservable();
  }

  public setActiveStep(step: number) {
    this.activeStep$.next(step);
  }

  public getActiveStep(): Observable<number> {
    return this.activeStep$.asObservable();
  }

  public clearBescheidDocumentFile(): void {
    this.updateBescheidDocumentFile(null);
  }

  public updateBescheidDocumentFile(uri: ResourceUri): void {
    this.getBescheidDocumentControl().patchValue(uri);
  }

  private getBescheidDocumentControl(): FormControl {
    return <FormControl>this.form.controls[BescheidenFormService.FIELD_BESCHEID_DOCUMENT];
  }

  public isSendByManual(): Observable<boolean> {
    return this.sendByManual.asObservable();
  }

  public patchNachricht(documentResource: DocumentResource): void {
    const value = this.getValue();
    if (isEmpty(value.nachrichtSubject)) {
      if (isNotEmpty(documentResource.nachrichtSubject)) {
        this.setNachrichtSubject(documentResource.nachrichtSubject);
      } else {
        this.setNachrichtSubject(BescheidenFormService.BETREFF_DEFAULT);
      }
    }

    if (isEmpty(value.nachrichtText)) {
      if (isNotEmpty(documentResource.nachrichtText)) {
        this.setNachrichtText(documentResource.nachrichtText);
      } else {
        this.setNachrichtText(this.buildNachrichtentext());
      }
    }
  }

  public clearNachricht(): void {
    this.setNachrichtSubject(EMPTY_STRING);
    this.setNachrichtText(EMPTY_STRING);
  }

  private setNachrichtSubject(value: string): void {
    this.form.controls[BescheidenFormService.FIELD_NACHRICHT_SUBJECT].patchValue(value);
  }

  private setNachrichtText(value: string): void {
    this.form.controls[BescheidenFormService.FIELD_NACHRICHT_TEXT].patchValue(value);
  }

  public setSendBy(sendBy: BescheidSendBy): void {
    this.form.controls[BescheidenFormService.FIELD_SEND_BY].patchValue(sendBy);
  }

  buildNachrichtentext(): string {
    return this.getValue().bewilligt ?
        BescheidenFormService.NACHRICHTENTEXT_BEWILLIGT_DEFAULT
      : BescheidenFormService.NACHRICHTENTEXT_ABGELEHNT_DEFAULT;
  }

  public getShowMissingBescheidDocumentError(): Observable<boolean> {
    return combineLatest([
      this.showMissingBescheidDocumentError$.asObservable(),
      this.getBescheidChanges(),
    ]).pipe(
      map(
        ([showMissingBescheidDocumentError, bescheid]) =>
          showMissingBescheidDocumentError && isEmpty(bescheid.bescheidDocument),
      ),
    );
  }

  public validateBescheidDocumentExists(): boolean {
    this.showMissingBescheidDocumentError$.next(isEmpty(this.getValue().bescheidDocument));
    return isNotEmpty(this.getValue().bescheidDocument);
  }
}
