import { BescheidLinkRel, BescheidResource, BescheidService } from '@alfa-client/bescheid-shared';
import { BinaryFile2ContainerComponent } from '@alfa-client/binary-file';
import { CommandResource } from '@alfa-client/command-shared';
import {
  ConvertApiErrorToErrorMessagesPipe,
  createEmptyStateResource,
  createStateResource,
  StateResource,
} from '@alfa-client/tech-shared';
import { existsAsHtmlElement, Mock, mock, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { getUrl } from '@ngxp/rest';
import { AttachmentComponent, AttachmentContainerComponent } from '@ods/system';
import { createBescheidResource } from 'libs/bescheid-shared/src/test/bescheid';
import { createBinaryFileResource } from 'libs/binary-file-shared/test/binary-file';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createApiError } from 'libs/tech-shared/test/error';
import { MockComponent, MockPipe } from 'ng-mocks';
import {
  createCommandErrorStateResource,
  createCommandResource,
  createCommandStateResource,
} from '../../../../../../../command-shared/test/command';
import { VorgangDetailBescheidenResultDokumentComponent } from './vorgang-detail-bescheiden-result-dokument.component';

describe('VorgangDetailBescheidenResultDokumentComponent', () => {
  let component: VorgangDetailBescheidenResultDokumentComponent;
  let fixture: ComponentFixture<VorgangDetailBescheidenResultDokumentComponent>;

  const createBescheidDocumentAttachment: string = getDataTestIdOf(
    'create-bescheid-document-attachment',
  );
  const uploadBescheidDocumentError: string = getDataTestIdOf(
    'upload-bescheid-document-error-true',
  );
  const missingBescheidDocumentErrorMessage: string = getDataTestIdOf(
    'missing-bescheid-document-error-message',
  );

  let bescheidService: Mock<BescheidService>;

  beforeEach(async () => {
    bescheidService = mock(BescheidService);

    await TestBed.configureTestingModule({
      declarations: [
        VorgangDetailBescheidenResultDokumentComponent,
        MockComponent(BinaryFile2ContainerComponent),
        MockComponent(AttachmentComponent),
        MockComponent(AttachmentContainerComponent),
        MockPipe(ConvertApiErrorToErrorMessagesPipe),
      ],
      providers: [
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangDetailBescheidenResultDokumentComponent);
    component = fixture.componentInstance;
    component.bescheidDocumentFile = createStateResource(createBinaryFileResource());
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('render', () => {
    it('should show error message for missing bescheid document', () => {
      component.showMissingBescheidDocumentError = true;

      fixture.detectChanges();

      existsAsHtmlElement(fixture, missingBescheidDocumentErrorMessage);
    });

    it('should not show error message for missing bescheid document', () => {
      component.showMissingBescheidDocumentError = false;

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, missingBescheidDocumentErrorMessage);
    });

    describe('create bescheid document ods-attachment', () => {
      it('should be shown if error exists', () => {
        component.createBescheidDocumentInProgress = createCommandErrorStateResource();

        fixture.detectChanges();

        existsAsHtmlElement(fixture, createBescheidDocumentAttachment);
      });

      it('should be shown while loading', () => {
        component.createBescheidDocumentInProgress = createStateResource(
          createCommandResource(),
          true,
        );

        fixture.detectChanges();

        existsAsHtmlElement(fixture, createBescheidDocumentAttachment);
      });

      it('should be hidden on loaded without error', () => {
        component.createBescheidDocumentInProgress = createCommandStateResource();

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, createBescheidDocumentAttachment);
      });
    });
  });

  describe('handle bescheid document', () => {
    it('should call service to load bescheid document by uri if link is present', () => {
      const bescheid: BescheidResource = createBescheidResource([
        BescheidLinkRel.BESCHEID_DOCUMENT,
      ]);

      component.handleBescheidDocument(bescheid);

      expect(bescheidService.loadBescheidDocumentByUri).toHaveBeenCalledWith(
        getUrl(bescheid, BescheidLinkRel.BESCHEID_DOCUMENT),
      );
    });

    it('should NOT call service if link is missing', () => {
      component.handleBescheidDocument(createBescheidResource());

      expect(bescheidService.loadBescheidDocumentByUri).not.toHaveBeenCalled();
    });
  });

  describe('upload bescheid document error', () => {
    it('should be shown if error exists', () => {
      component.uploadBescheidDocumentInProgress = { loading: false, error: createApiError() };

      fixture.detectChanges();

      existsAsHtmlElement(fixture, uploadBescheidDocumentError);
    });

    it('should be hidden on non error', () => {
      component.uploadBescheidDocumentInProgress = { loading: false };

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, uploadBescheidDocumentError);
    });
  });

  describe('set create bescheid document in progress', () => {
    beforeEach(() => {
      component.handleCreateBescheidDocumentCommandError = jest.fn();
    });

    it('should set create document in progress', () => {
      const commandStateResource: StateResource<CommandResource> = createEmptyStateResource();

      component.createBescheidDocumentInProgress = commandStateResource;

      expect(component.createDocumentInProgess).toBe(commandStateResource);
    });

    it('should handle command error', () => {
      const commandStateResource: StateResource<CommandResource> = createEmptyStateResource();

      component.createBescheidDocumentInProgress = commandStateResource;

      expect(component.handleCreateBescheidDocumentCommandError).toHaveBeenCalledWith(
        commandStateResource,
      );
    });
  });

  describe('handleCreateBescheidDocumentCommandError', () => {
    it('should set has error', () => {
      component.handleCreateBescheidDocumentCommandError(createCommandErrorStateResource());

      expect(component.hasCreateDocumentError).toBeTruthy();
    });

    it('should not set has error', () => {
      component.handleCreateBescheidDocumentCommandError(createCommandStateResource());

      expect(component.hasCreateDocumentError).toBeFalsy();
    });

    it('should not set has error on empty state resource', () => {
      component.handleCreateBescheidDocumentCommandError(createEmptyStateResource());

      expect(component.hasCreateDocumentError).toBeFalsy();
    });

    it('should set error messages', () => {
      component.handleCreateBescheidDocumentCommandError(createCommandErrorStateResource());

      expect(component.createDocumentErrorMessages).toEqual([
        VorgangDetailBescheidenResultDokumentComponent.CREATE_DOCUMENT_ERROR_MESSAGE,
      ]);
    });

    it('should not set error messages', () => {
      component.handleCreateBescheidDocumentCommandError(createCommandStateResource());

      expect(component.createDocumentErrorMessages).toEqual([]);
    });
  });
});
