/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { NavigationService } from '@alfa-client/navigation-shared';
import {
  ConvertForDataTestPipe,
  EMPTY_STRING,
  HasLinkPipe,
  ToEmbeddedResourcesPipe,
  createEmptyStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import {
  dispatchEventFromFixture,
  getElementFromFixture,
  mock,
  useFromMock,
} from '@alfa-client/test-utils';
import { SpinnerComponent } from '@alfa-client/ui';
import { VorgangHeaderLinkRel, VorgangListService } from '@alfa-client/vorgang-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import {
  ReactiveFormsModule,
  UntypedFormBuilder,
  UntypedFormControl,
  UntypedFormGroup,
} from '@angular/forms';
import { MatAutocompleteModule } from '@angular/material/autocomplete';
import { MatButton } from '@angular/material/button';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatIcon } from '@angular/material/icon';
import { MatInputModule } from '@angular/material/input';
import { NoopAnimationsModule } from '@angular/platform-browser/animations';
import { NavigationEnd, Router, RouterEvent } from '@angular/router';
import { RouterTestingModule } from '@angular/router/testing';
import { getDataTestClassOf, getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createVorgangListResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { BehaviorSubject, ReplaySubject, Subject, of } from 'rxjs';
import { VorgangSearchAutocompleteOptionsContentComponent } from './vorgang-search-autocomplete-options-content/vorgang-search-autocomplete-options-content.component';
import { VorgangSearchClearButtonComponent } from './vorgang-search-clear-button/vorgang-search-clear-button.component';
import { VorgangSearchComponent } from './vorgang-search.component';
import { VorgangSearchFormService } from './vorgang-search.formservice';

describe('VorgangSearchComponent', () => {
  let component: VorgangSearchComponent;
  let fixture: ComponentFixture<VorgangSearchComponent>;
  let searchFormService: VorgangSearchFormService;

  const vorgangListService = mock(VorgangListService);
  const navigationService = mock(NavigationService);

  const searchStringSubj: Subject<string> = new BehaviorSubject(EMPTY_STRING);

  const routerEvents: ReplaySubject<RouterEvent> = new ReplaySubject<RouterEvent>();
  const routerMock = {
    navigate: jest.fn(),
    url: '/',
    events: routerEvents.asObservable(),
  };

  const searchPreviewOption: string = getDataTestClassOf('search-preview-option');
  const searchClearButton: string = getDataTestIdOf('vorgang-search-clear-button');

  beforeEach(async () => {
    vorgangListService.getSearchString.mockReturnValue(searchStringSubj);

    searchFormService = new VorgangSearchFormService(
      new UntypedFormBuilder(),
      useFromMock(navigationService),
      useFromMock(vorgangListService),
    );

    await TestBed.configureTestingModule({
      declarations: [
        VorgangSearchComponent,
        MatIcon,
        ToEmbeddedResourcesPipe,
        HasLinkPipe,
        ConvertForDataTestPipe,
        MockComponent(VorgangSearchAutocompleteOptionsContentComponent),
        MockComponent(VorgangSearchClearButtonComponent),
        MockComponent(SpinnerComponent),
      ],
      imports: [
        MatFormFieldModule,
        MatInputModule,
        ReactiveFormsModule,
        NoopAnimationsModule,
        RouterTestingModule,
        MatAutocompleteModule,
      ],
      providers: [
        UntypedFormBuilder,
        {
          provide: VorgangSearchFormService,
          useValue: searchFormService,
        },
        {
          provide: VorgangListService,
          useValue: vorgangListService,
        },
        {
          provide: NavigationService,
          useValue: navigationService,
        },
        {
          provide: Router,
          useValue: routerMock,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(VorgangSearchComponent);
    component = fixture.componentInstance;
    component.vorgangSearchPreviewList = createEmptyStateResource();
    navigationService.urlChanged.mockReturnValue(of({}));
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe.skip('FIXME: search preview option', () => {
    it('should hide on non existing link', () => {
      component.vorgangSearchPreviewList = createStateResource(createVorgangListResource());

      fixture.detectChanges();
      const button = getElementFromFixture(fixture, searchPreviewOption);

      expect(button).not.toBeInstanceOf(HTMLElement);
    });

    it('should shown on existing link', () => {
      component.vorgangSearchPreviewList = createStateResource(
        createVorgangListResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG]),
      );

      fixture.detectChanges();
      const button = getElementFromFixture(fixture, searchPreviewOption);

      expect(button).toBeInstanceOf(HTMLElement);
    });
  });

  describe('ngOnInit', () => {
    it('should set focus if path ends with "/search"', () => {
      component.focus = jest.fn();
      routerEvents.next(new NavigationEnd(1, '/alle/search', ''));

      component.ngOnInit();

      expect(component.focus).toHaveBeenCalled();
    });

    it('should not set focus if path does not end with "/search"', () => {
      component.focus = jest.fn();
      routerEvents.next(new NavigationEnd(1, '/alle', ''));

      component.ngOnInit();

      expect(component.focus).toHaveBeenCalled();
    });
  });

  describe('search clear button', () => {
    beforeEach(() => {
      searchStringSubj.next('test');
      fixture.detectChanges();
    });

    it('should emit clearVorgangSearchPreviewList event', () => {
      jest.spyOn(component.clearVorgangSearchPreviewList, 'emit');

      // Workaround: Timingprobleme entfernen den Clear-Button zu früh,
      // seit debounceTime() in VorgangSearchFormService::subscribeToValueChanges() vor setHasSearchString() kommt.
      // dispatchEventFromFixture(fixture, searchClearButton, 'clear');
      component.handleSearchClearButton();

      expect(component.clearVorgangSearchPreviewList.emit).toHaveBeenCalled();
    });

    it.skip('FIXME: should call searchFormService.clearSearchField', () => {
      searchFormService.clearSearchField = jest.fn();

      dispatchEventFromFixture(fixture, searchClearButton, 'clear');

      expect(searchFormService.clearSearchField).toHaveBeenCalled();
    });

    it('should set focus to input', () => {
      jest.spyOn(component.searchInput.nativeElement, 'focus');

      // Workaround: Timingprobleme entfernen den Clear-Button zu früh,
      // seit debounceTime() in VorgangSearchFormService::subscribeToValueChanges() vor setHasSearchString() kommt.
      // dispatchEventFromFixture(fixture, searchClearButton, 'clear');
      component.handleSearchClearButton();

      expect(component.searchInput.nativeElement.focus).toHaveBeenCalled();
    });

    it('should call handleSearchClearButton', () => {
      component.handleSearchClearButton = jest.fn();

      // Workaround: Timingprobleme entfernen den Clear-Button zu früh,
      // seit debounceTime() in VorgangSearchFormService::subscribeToValueChanges() vor setHasSearchString() kommt.
      // dispatchEventFromFixture(fixture, searchClearButton, 'clear');
      component.handleSearchClearButton();

      expect(component.handleSearchClearButton).toHaveBeenCalled();
    });
  });

  describe('submit', () => {
    it('should not navigate to search if search field is empty', () => {
      const submitSpy = jest.spyOn(NavigationService.prototype, 'search');

      fixture.detectChanges();
      component.submit();

      expect(submitSpy).not.toHaveBeenCalled();
    });

    it('should store entered search value', () => {
      jest.spyOn(VorgangSearchFormService.prototype, 'getValue').mockReturnValue('Gewerbe');

      component.submit();

      expect(component.previouslyEnteredSearchValue).toEqual('Gewerbe');
    });

    it('should submit form', () => {
      const submitSpy = jest.spyOn(VorgangSearchFormService.prototype, 'submit');

      component.submit();

      expect(submitSpy).toHaveBeenCalled();
    });

    it('should remove focus from search INPUT', () => {
      component.blur = jest.fn();

      component.submit();

      expect(component.blur).toHaveBeenCalled();
    });
  });

  describe('focusIn', () => {
    it('should store current form value', () => {
      jest.spyOn(VorgangSearchFormService.prototype, 'getValue').mockReturnValue('Gewerbe');

      component.focusIn();

      expect(component.previouslyEnteredSearchValue).toEqual('Gewerbe');
    });
  });

  describe('focusOut', () => {
    function createSearchSubmitMockButton(nativeElement: any): MatButton {
      return {
        _elementRef: {
          nativeElement: nativeElement,
        },
      } as MatButton;
    }

    function createMockFormGroup(searchFieldFormControl: UntypedFormControl): UntypedFormGroup {
      return new UntypedFormGroup({
        search: searchFieldFormControl,
      });
    }

    it('should set new form value when focus out to search button (Lupe)', () => {
      const lupeElement = {};
      component.previouslyEnteredSearchValue = 'Gewerbe';
      component.searchSubmitButton = createSearchSubmitMockButton(lupeElement);
      const searchFieldFormControl = new UntypedFormControl('Gewe');
      component.formService.form = createMockFormGroup(searchFieldFormControl);

      component.focusOut({ relatedTarget: lupeElement } as FocusEvent);

      expect(searchFieldFormControl.value).toEqual('Gewe');
    });

    it('should set previous value', () => {
      component.previouslyEnteredSearchValue = 'Gewe';
      component.searchSubmitButton = createSearchSubmitMockButton(null);
      const searchFieldFormControl = new UntypedFormControl('Gewerbe');
      component.formService.form = createMockFormGroup(searchFieldFormControl);

      component.focusOut({ relatedTarget: {} } as FocusEvent);

      expect(searchFieldFormControl.value).toEqual('Gewe');
    });
  });
});
