package de.ozgcloud.alfa.bescheid;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;

class BescheidByBeschiedenAmComparatorTest {

	@Spy
	private BescheidByBeschiedenAmComparator comparator;

	@Nested
	class TestCompare {
		int expectedResult = -1;

		@Nested
		class TestWithDifferentBeschiedenAm {
			private final Bescheid olderBescheid = BescheidTestFactory.create();
			private final Bescheid recentBescheid = BescheidTestFactory.create();

			@BeforeEach
			void setUpMock() {
				doReturn(expectedResult).when(comparator).compareByBeschiedenAm(olderBescheid, recentBescheid);
			}

			@Test
			void shouldCompareByBeschiedenAm() {
				comparator.compare(olderBescheid, recentBescheid);

				verify(comparator).compareByBeschiedenAm(olderBescheid, recentBescheid);
			}

			@Test
			void shouldNotCompareBySentAt() {
				comparator.compare(olderBescheid, recentBescheid);

				verify(comparator, never()).compareBySentAt(any(), any());
			}

			@Test
			void shouldReturnCompareByBeschiedenAmResult() {
				var compareResult = comparator.compare(olderBescheid, recentBescheid);

				assertThat(compareResult).isEqualTo(expectedResult);
			}
		}

		@Nested
		class TestWithSameBeschiedenAm {
			private final Bescheid olderBescheid = BescheidTestFactory.create();
			private final Bescheid recentBescheid = BescheidTestFactory.create();

			@BeforeEach
			void setUpMock() {
				doReturn(0).when(comparator).compareByBeschiedenAm(olderBescheid, recentBescheid);
			}

			@Test
			void shouldCompareByBeschiedenAm() {
				comparator.compare(olderBescheid, recentBescheid);

				verify(comparator).compareByBeschiedenAm(olderBescheid, recentBescheid);
			}

			@Test
			void shouldCompareBySentAt() {
				comparator.compare(olderBescheid, recentBescheid);

				verify(comparator).compareBySentAt(any(), any());
			}

			@Test
			void shouldReturnCompareBySentAtResult() {
				doReturn(expectedResult).when(comparator).compareBySentAt(olderBescheid, recentBescheid);

				var compareResult = comparator.compare(olderBescheid, recentBescheid);

				assertThat(compareResult).isEqualTo(expectedResult);
			}
		}
	}

	@Nested
	class TestCompareByBeschiedenAm {
		private final Bescheid olderBescheid = BescheidTestFactory.createBuilder().beschiedenAm("2024-05-26").build();
		private final Bescheid recentBescheid = BescheidTestFactory.createBuilder().beschiedenAm("2024-05-27").build();

		@Test
		void shouldReturnNegative() {
			var result = comparator.compareByBeschiedenAm(olderBescheid, recentBescheid);

			assertThat(result).isNegative();
		}

		@Test
		void shouldReturnPositive() {
			var result = comparator.compareByBeschiedenAm(recentBescheid, olderBescheid);

			assertThat(result).isPositive();
		}

		@Test
		void shouldReturnZero() {
			var result = comparator.compareByBeschiedenAm(olderBescheid, olderBescheid);

			assertThat(result).isZero();
		}
	}

	@Nested
	class TestComapreBySentAt {

		private final Bescheid olderBescheid = BescheidTestFactory.createBuilder()
				.sentInfo(SentInfoTestFactory.create())
				.build();
		private final Bescheid recentBescheid = BescheidTestFactory.createBuilder()
				.sentInfo(SentInfoTestFactory.createBuilder().sentAt(SentInfoTestFactory.SENT_AT.plusHours(1)).build())
				.build();

		@Test
		void shouldReturnNegative() {
			var result = comparator.compareBySentAt(olderBescheid, recentBescheid);

			assertThat(result).isNegative();
		}

		@Test
		void shouldReturnPositive() {
			var result = comparator.compareBySentAt(recentBescheid, olderBescheid);

			assertThat(result).isPositive();
		}

		@Test
		void shouldReturnZero() {
			var result = comparator.compareBySentAt(olderBescheid, olderBescheid);

			assertThat(result).isZero();
		}
	}
}
