/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.binaryfile;

import static de.ozgcloud.alfa.JwtTokenUtil.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;

import de.ozgcloud.alfa.common.ValidationMessageCodes;
import de.ozgcloud.alfa.common.downloadtoken.DownloadTokenController;
import de.ozgcloud.alfa.common.downloadtoken.DownloadTokenProperties;
import de.ozgcloud.alfa.common.downloadtoken.DownloadTokenTestFactory;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.common.test.ITCase;
import io.jsonwebtoken.JwtBuilder;

@AutoConfigureMockMvc
@WithMockUser
@ITCase
class BinaryFileControllerITCase {

	@SpyBean
	private BinaryFileController controller;
	@MockBean
	private BinaryFileRemoteService remoteService;
	@Autowired
	private MockMvc mockMvc;
	@Autowired
	private DownloadTokenProperties downloadTokenProperties;

	private FileId fileId = FileId.createNew();

	@Nested
	class TestGetFile {

		@Nested
		class TestDragAndDropFlow {

			private JwtBuilder tokenBuilder = DownloadTokenTestFactory.createTokenBuilder(downloadTokenProperties.getSecret(),
					downloadTokenProperties.getValidity());

			@BeforeEach
			void init() {
				when(remoteService.getFile(any())).thenReturn(OzgFileTestFactory.create());
			}

			@Test
			void shouldGet403WhenNoFileId() throws Exception {
				setTokenToSecuriyContext(tokenBuilder.compact());

				performRequest().andExpect(status().isForbidden());
			}

			@Test
			void shouldGet403WhenWrongFileId() throws Exception {
				setTokenToSecuriyContext(tokenBuilder.addClaims(createClaims(FileId.createNew())).compact());

				performRequest().andExpect(status().isForbidden());
			}

			@Test
			void shouldGetFile() throws Exception {
				setTokenToSecuriyContext(tokenBuilder.addClaims(createClaims(fileId)).compact());

				performRequest().andExpect(status().isOk());
			}

			private Map<String, Object> createClaims(FileId fileId) {
				return new HashMap<>(Map.of(
						FIRSTNAME_CLAIM, UserProfileTestFactory.FIRSTNAME,
						LASTNAME_CLAIM, UserProfileTestFactory.LASTNAME,
						ROLE_CLAIM, List.of(),
						FILEID_CLAIM, fileId.toString()));
			}

			void setTokenToSecuriyContext(String token) throws Exception {
				mockMvc.perform(get(DownloadTokenController.DOWNLOAD_TOKEN_PATH + "?" + DownloadTokenController.PARAM_TOKEN + "=" + token)
								.with(csrf()))
						.andExpect(status().isOk());
			}
		}

		@Nested
		class TestDefaultFlow {

			@Mock
			private Authentication authentication;

			@BeforeEach
			void init() {
				when(remoteService.getFile(any())).thenReturn(OzgFileTestFactory.create());
				SecurityContextHolder.getContext().setAuthentication(authentication);
			}

			@Test
			void shouldGet403WhenUnautorised() throws Exception {
				when(authentication.isAuthenticated()).thenReturn(Boolean.FALSE);

				performRequest().andExpect(status().isForbidden());
			}

			@Test
			void shouldGetFile() throws Exception {
				when(authentication.isAuthenticated()).thenReturn(Boolean.TRUE);

				performRequest().andExpect(status().isOk());
			}
		}

		ResultActions performRequest() throws Exception {
			return mockMvc.perform(get(BinaryFileController.PATH + "/" + fileId.toString()).with(csrf()));
		}
	}

	@Nested
	class TestUploadFile {

		@Nested
		class TestFileContentType {

			@Test
			void shouldReturnValidationMessage() throws Exception {
				performRequest(createFile(MediaType.TEXT_PLAIN)).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.issues.length()").value(1))
						.andExpect(jsonPath("$.issues.[0].messageCode").value(ValidationMessageCodes.FIELD_FILE_CONTENT_TYPE_INVALID));
			}

			@Test
			void shouldAcceptConfiguredContentType() throws Exception {
				when(remoteService.uploadFile(any())).thenReturn(CompletableFuture.completedFuture(FileId.createNew()));

				performRequest(createFile(MediaType.APPLICATION_PDF)).andExpect(status().isOk());
			}

			private ResultActions performRequest(MockMultipartFile file) throws Exception {
				return mockMvc.perform(multipart(
						String.format("%s/%s/%s/file", BinaryFileController.PATH, VorgangHeaderTestFactory.ID,
								UploadBinaryFileTestFactory.FIELD)).file(
						file).with(csrf()));
			}

			private MockMultipartFile createFile(MediaType contentType) {
				return new MockMultipartFile("file", "hello.txt", contentType.toString(), "Hello, World!".getBytes());
			}
		}
	}
}
