/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.errorhandling;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.Map;

import jakarta.validation.ConstraintViolationException;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Spy;
import org.springframework.security.access.AccessDeniedException;

import de.ozgcloud.alfa.common.binaryfile.DynamicViolationParameter;
import de.ozgcloud.alfa.common.command.LegacyOrder;
import de.ozgcloud.common.errorhandling.TechnicalException;

class ExceptionControllerTest {

	@Spy
	@InjectMocks
	private ExceptionController exceptionController;

	@Nested
	class TestHandleFunctionalException {

		private final FunctionalException exception = new FunctionalException(() -> ExceptionTestFactory.MESSAGE_CODE);

		@Test
		void shouldHaveMessageCode() {
			var error = handleException();

			assertThat(error.getIssues()).hasSize(1);
			assertThat(error.getIssues().get(0).getMessageCode()).isEqualTo(ExceptionTestFactory.MESSAGE_CODE);
		}

		@Test
		void shouldHaveMessage() {
			var error = handleException();

			assertThat(error.getIssues()).hasSize(1);
			assertThat(error.getIssues().get(0).getMessage()).isEqualTo(exception.getMessage());
			assertThat(error.getIssues().get(0).getMessage()).contains(exception.getExceptionId());
		}

		@Test
		void shouldHaveExceptionId() {
			var error = handleException();

			assertThat(error.getIssues()).hasSize(1);
			assertThat(error.getIssues().get(0).getExceptionId()).isEqualTo(exception.getExceptionId());
		}

		private ApiError handleException() {
			return exceptionController.handleFunctionalException(exception);
		}
	}

	@Nested
	class TestHandleAccessDeniedException {

		private final AccessDeniedException exception = new AccessDeniedException(ExceptionTestFactory.MESSAGE);

		@BeforeEach
		void mockExceptionId() {
			doReturn(ExceptionTestFactory.EXCEPTION_ID).when(exceptionController).createExceptionId();
		}

		@Test
		void shouldHaveMessageCode() {
			var error = handleException();

			assertThat(error.getIssues()).hasSize(1);
			assertThat(error.getIssues().get(0).getMessageCode()).isEqualTo(ExceptionController.ACCESS_DENIED_MESSAGE_CODE);
		}

		@Test
		void shouldHaveMessage() {
			var error = handleException();

			assertThat(error.getIssues()).hasSize(1);
			assertThat(error.getIssues().get(0).getMessage()).contains(exception.getMessage());
			assertThat(error.getIssues().get(0).getMessage()).contains(ExceptionTestFactory.EXCEPTION_ID);
		}

		@Test
		void shouldHaveExceptionId() {
			var error = handleException();

			assertThat(error.getIssues()).hasSize(1);
			assertThat(error.getIssues().get(0).getExceptionId()).isEqualTo(ExceptionTestFactory.EXCEPTION_ID);
		}

		private ApiError handleException() {
			return exceptionController.handleAccessDeniedException(exception);
		}
	}

	@Nested
	class TestHandleResourceNotFoundException {

		private final ResourceNotFoundException exception = new ResourceNotFoundException(String.class, 42L);

		@Test
		void shouldHaveMessageCode() {
			var error = handleException();

			assertThat(error.getIssues()).hasSize(1);
			assertThat(error.getIssues().get(0).getMessageCode()).isEqualTo(ExceptionController.RESOURCE_NOT_FOUNT_MESSAGE_CODE);
		}

		@Test
		void shouldHaveMessage() {
			var error = handleException();

			assertThat(error.getIssues()).hasSize(1);
			assertThat(error.getIssues().get(0).getMessage()).isEqualTo(exception.getMessage());
			assertThat(error.getIssues().get(0).getMessage()).contains(exception.getExceptionId());
		}

		@Test
		void shouldHaveExceptionId() {
			var error = handleException();

			assertThat(error.getIssues()).hasSize(1);
			assertThat(error.getIssues().get(0).getExceptionId()).isEqualTo(exception.getExceptionId());
		}

		private ApiError handleException() {
			return exceptionController.handleResourceNotFoundException(exception);
		}
	}

	@Nested
	class TestContraintValidationException {

		private final ConstraintViolationException exception = new ConstraintViolationException(ExceptionTestFactory.MESSAGE,
				Collections.singleton(ExceptionTestFactory.buildMockedConstraintViolation()));

		@BeforeEach
		void mockExceptionId() {
			doReturn(ExceptionTestFactory.EXCEPTION_ID).when(exceptionController).createExceptionId();
		}

		@Test
		void shouldHaveField() {
			var error = handleException();

			assertThat(error.getIssues()).hasSize(1);
			assertThat(error.getIssues().get(0).getField()).isEqualTo(ExceptionTestFactory.PATH_FIELD);
		}

		@Test
		void shouldHaveMessageCode() {
			var error = handleException();

			assertThat(error.getIssues()).hasSize(1);
			assertThat(error.getIssues().get(0).getMessageCode()).isEqualTo(ExceptionTestFactory.MESSAGE_CODE);
		}

		@Test
		void shouldHaveMessage() {
			var error = handleException();

			assertThat(error.getIssues()).hasSize(1);
			assertThat(error.getIssues().get(0).getMessage()).isEqualTo(ExceptionTestFactory.MESSAGE);
		}

		@Test
		void shouldHaveExceptionId() {
			var error = handleException();

			assertThat(error.getIssues().get(0).getExceptionId()).isEqualTo(ExceptionTestFactory.EXCEPTION_ID);
		}

		@Test
		void shouldHaveIssueParameter() {
			var error = handleException();

			var issueParameter = error.getIssues().get(0).getParameters().get(0);
			assertThat(issueParameter.getName()).isEqualTo(ExceptionTestFactory.PARAM_NAME);
			assertThat(issueParameter.getValue()).isEqualTo(ExceptionTestFactory.PARAM_DYNAMIC_VALUE);
		}

		@Nested
		class TestWithDynamicPayload {

			private final DynamicViolationParameter dynamicViolationParameter = DynamicViolationParameter.builder()
					.map(Map.of(ExceptionTestFactory.PARAM_NAME, ExceptionTestFactory.PARAM_DYNAMIC_VALUE)).build();
			private final ConstraintViolationException exceptionWithDynamicPayload = new ConstraintViolationException(ExceptionTestFactory.MESSAGE,
					Collections.singleton(ExceptionTestFactory.buildMockedConstraintViolationWithDynamicPayload(dynamicViolationParameter)));

			@Test
			void shouldHaveReplacedParams() {
				var error = handleExceptionWithDynamicPayload();

				var issueParameter = error.getIssues().get(0).getParameters().get(0);
				assertThat(issueParameter.getName()).isEqualTo(ExceptionTestFactory.PARAM_NAME);
				assertThat(issueParameter.getValue()).isEqualTo(ExceptionTestFactory.PARAM_DYNAMIC_VALUE);
			}

			private ApiError handleExceptionWithDynamicPayload() {
				return exceptionController.handleConstraintViolationException(exceptionWithDynamicPayload);
			}
		}

		private ApiError handleException() {
			return exceptionController.handleConstraintViolationException(exception);
		}
	}

	@Nested
	class TestHandleTechnicalException {

		private final TechnicalException exception = new TechnicalException(ExceptionTestFactory.MESSAGE, new Throwable());

		@Test
		void shouldHaveMessageCode() {
			var error = handleException();

			assertThat(error.getIssues()).hasSize(1);
			assertThat(error.getIssues().get(0).getMessageCode()).isEqualTo(ExceptionController.RUNTIME_MESSAGE_CODE);
		}

		@Test
		void shouldHaveMessage() {
			var error = handleException();

			assertThat(error.getIssues()).hasSize(1);
			assertThat(error.getIssues().get(0).getMessage()).isEqualTo(exception.getMessage());
			assertThat(error.getIssues().get(0).getMessage()).contains(exception.getExceptionId());
		}

		@Test
		void shouldHaveExceptionId() {
			var error = handleException();

			assertThat(error.getIssues()).hasSize(1);
			assertThat(error.getIssues().get(0).getExceptionId()).isEqualTo(exception.getExceptionId());
		}

		private ApiError handleException() {
			return exceptionController.handleTechnicalException(exception);
		}
	}

	@Nested
	class TestHandleRuntimeException {

		private final RuntimeException exception = new RuntimeException(ExceptionTestFactory.MESSAGE);

		@BeforeEach
		void mockExceptionId() {
			doReturn(ExceptionTestFactory.EXCEPTION_ID).when(exceptionController).createExceptionId();
		}

		@Test
		void shouldHaveMessageCode() {
			var error = handleException();

			assertThat(error.getIssues()).hasSize(1);
			assertThat(error.getIssues().get(0).getMessageCode()).isEqualTo(ExceptionController.RUNTIME_MESSAGE_CODE);
		}

		@Test
		void shouldHaveMessage() {
			var error = handleException();

			assertThat(error.getIssues()).hasSize(1);
			assertThat(error.getIssues().get(0).getMessage()).contains(exception.getMessage());
			assertThat(error.getIssues().get(0).getMessage()).contains(ExceptionTestFactory.EXCEPTION_ID);
		}

		@Test
		void shouldHaveExceptionId() {
			var error = handleException();

			assertThat(error.getIssues()).hasSize(1);
			assertThat(error.getIssues().get(0).getExceptionId()).isEqualTo(ExceptionTestFactory.EXCEPTION_ID);
		}

		private ApiError handleException() {
			return exceptionController.handleRuntimeException(exception);
		}
	}

	@DisplayName("Handle OrderNotAllowedException")
	@Nested
	class TestOrderNotAllowedException {

		private final OrderNotAllowedException exception = new OrderNotAllowedException(LegacyOrder.EDIT_WIEDERVORLAGE);

		@Test
		void shouldHaveMessage() {
			var apiError = exceptionController.handleFunctionalException(exception);

			var messageString = "Order 'EDIT_WIEDERVORLAGE' is not allowed. Expected 'VORGANG_LOESCHEN or LOESCH_ANFORDERUNG_ZURUECKNEHMEN'.";
			assertThat(apiError.getIssues().get(0).getMessage()).contains(messageString);
		}

		@Test
		void shouldHaveExceptionId() {
			var apiError = exceptionController.handleFunctionalException(exception);

			assertThat(apiError.getIssues()).hasSize(1).first().extracting(Issue::getExceptionId)
					.isEqualTo(exception.getExceptionId());
		}

		@Test
		void shouldHaveMessageCode() {
			var apiError = exceptionController.handleFunctionalException(exception);

			var messageCode = "loeschanforderung.order_not_allowed";
			assertThat(apiError.getIssues()).hasSize(1).first().extracting(Issue::getMessageCode).isEqualTo(messageCode);
		}
	}
}