package de.ozgcloud.alfa.loeschanforderung;

import static de.ozgcloud.alfa.common.command.CommandController.*;
import static org.assertj.core.api.Assertions.*;
import static org.hamcrest.CoreMatchers.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.nio.charset.StandardCharsets;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.hateoas.EntityModel;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItem;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemService;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemTestFactory;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandController;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandStatus;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.command.CreateCommand;
import de.ozgcloud.alfa.common.command.LegacyOrder;
import de.ozgcloud.alfa.common.command.StatusPatch;
import de.ozgcloud.alfa.common.errorhandling.ExceptionController;
import de.ozgcloud.alfa.vorgang.VorgangController;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;
import lombok.SneakyThrows;

class LoeschAnforderungCommandControllerTest {

	@Spy
	@InjectMocks
	private LoeschAnforderungCommandController controller;

	@Mock
	private LoeschAnforderungService loeschAnforderungService;

	@Mock
	private VorgangController vorgangController;

	@Mock
	private VorgangAttachedItemService vorgangAttachedItemService;

	@Mock
	private CommandController commandController;

	private MockMvc mockMvc;

	@BeforeEach
	void init() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).setControllerAdvice(new ExceptionController()).build();
	}

	@DisplayName("Create command")
	@Nested
	class TestCreateCommand {

		@Captor
		private ArgumentCaptor<CreateCommand> createCommandArgumentCaptor;

		@Nested
		class TestExecuteLoeschen {

			@BeforeEach
			void init() {
				when(loeschAnforderungService.vorgangLoeschen(any(), any())).thenReturn(CommandTestFactory.create());
			}

			@Test
			void shouldCallLoeschAnforderungService() throws Exception {
				doRequest(CommandOrder.VORGANG_LOESCHEN.name()).andExpect(status().isCreated());

				verify(loeschAnforderungService).vorgangLoeschen(createCommandArgumentCaptor.capture(), eq(LoeschAnforderungTestFactory.ID));
				assertThat(createCommandArgumentCaptor.getValue()).usingRecursiveComparison()
						.isEqualTo(CommandTestFactory.createCreateCommandBuilder().vorgangId(null).relationVersion(0L).relationId(null)
								.order(CommandOrder.VORGANG_LOESCHEN.name()).build());
			}

			@Test
			void shouldReturnLinkToCommand() throws Exception {
				var response = doRequest(CommandOrder.VORGANG_LOESCHEN.name());

				response.andExpect(header().stringValues("location", "http://localhost" + COMMANDS_PATH + "/" + CommandTestFactory.ID));
			}
		}

		@Nested
		class TestLoeschAnforderungZuruecknehmen {

			private VorgangWithEingang vorgang;
			private VorgangAttachedItem vorgangAttachedItem;

			@BeforeEach
			void init() {
				vorgang = VorgangWithEingangTestFactory.create();
				vorgangAttachedItem = VorgangAttachedItemTestFactory.create();

				when(vorgangAttachedItemService.getById(LoeschAnforderungTestFactory.ID)).thenReturn(vorgangAttachedItem);
				when(vorgangController.getVorgang(VorgangHeaderTestFactory.ID)).thenReturn(vorgang);
				when(loeschAnforderungService.zuruecknehmen(vorgangAttachedItem, vorgang)).thenReturn(CommandTestFactory.create());
			}

			@Test
			void shouldLoadVorgangAttachedItem() throws Exception {
				doRequest(CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN.name()).andExpect(status().isCreated());

				verify(vorgangAttachedItemService).getById(LoeschAnforderungTestFactory.ID);
			}

			@Test
			void shouldLoadVorgang() throws Exception {
				doRequest(CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN.name()).andExpect(status().isCreated());

				verify(vorgangController).getVorgang(LoeschAnforderungTestFactory.VORGANG_ID);
			}

			@Test
			void shouldCallLoeschAnforderungService() throws Exception {
				doRequest(CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN.name()).andExpect(status().isCreated());

				verify(loeschAnforderungService).zuruecknehmen(vorgangAttachedItem, vorgang);
			}

			@Test
			void shouldReturnLinkToCommand() throws Exception {
				var response = doRequest(CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN.name());

				response.andExpect(header().stringValues("location", "http://localhost" + COMMANDS_PATH + "/" + CommandTestFactory.ID));
			}
		}

		@Test
		void shouldThrowOrderNotAllowedException() throws Exception {
			var message = "Order 'CREATE_KOMMENTAR' is not allowed. Expected 'VORGANG_LOESCHEN or LOESCH_ANFORDERUNG_ZURUECKNEHMEN'.";

			doRequest(LegacyOrder.CREATE_KOMMENTAR).andExpect(status().isBadRequest())
					.andExpect(jsonPath("issues[0].field").isEmpty())
					.andExpect(jsonPath("issues[0].exceptionId").isNotEmpty())
					.andExpect(jsonPath("issues[0].messageCode").value("loeschanforderung.order_not_allowed"))
					.andExpect(jsonPath("issues[0].message").value(containsString(message)));
		}

		private ResultActions doRequest(String commandOrderString) throws Exception {
			return mockMvc.perform(post(LoeschAnforderungCommandController.BASE_PATH, LoeschAnforderungTestFactory.ID)
					.content(CommandTestFactory.buildCreateVorgangCommandContent(commandOrderString))
					.contentType(MediaType.APPLICATION_JSON)
					.characterEncoding(StandardCharsets.UTF_8.name()));
		}
	}

	@DisplayName("Revoke")
	@Nested
	class TestRevoke {

		@DisplayName("on revoke command status patch")
		@Nested
		class TestOnRevokedStatus {

			private static final CommandStatus STATUS = CommandStatus.REVOKED;

			@Test
			void shouldGetCommand() {
				when(commandController.getById(any())).thenReturn(EntityModel.of(CommandTestFactory.create()));

				doRequest(STATUS);

				verify(commandController).getById(CommandTestFactory.ID);
			}

			@DisplayName("with VORGANG_ZUM_LOESCHEN_MARKIEREN order")
			@Nested
			class TestWithVorgangZumLoeschenMarkierenOrder {

				private final Command command = CommandTestFactory.createBuilder().order(CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN.name())
						.build();

				@BeforeEach
				void mockCommandController() {
					when(commandController.getById(any())).thenReturn(EntityModel.of(command));
					doReturn(ResponseEntity.ok().build()).when(controller).handleVorgangZumLoeschenMarkierenRevokeCommand(any(), any(), any());
				}

				@SneakyThrows
				@Test
				void shouldHandleVorgangZumLoeschenMarkierenRevokeCommand() {
					doRequest(STATUS).andExpect(status().isOk());

					verify(controller).handleVorgangZumLoeschenMarkierenRevokeCommand(eq(LoeschAnforderungTestFactory.ID), eq(command), any());
				}
			}

			@DisplayName("with LOESCH_ANFORDERUNG_ZURUECKNEHMEN order")
			@Nested
			class TestWithLoeschAnforderungZuruecknehmenOrder {

				private final Command command = CommandTestFactory.createBuilder().order(CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN.name())
						.build();

				@BeforeEach
				void mockCommandController() {
					when(commandController.getById(any())).thenReturn(EntityModel.of(command));
					doReturn(ResponseEntity.ok().build()).when(controller).handleLoeschAnforderungZuruecknehmenRevokeCommand(any(), any());
				}

				@Test
				void shouldHandleLoeschAnforderungZuruecknehmenRevokeCommand() {
					doRequest(STATUS);

					verify(controller).handleLoeschAnforderungZuruecknehmenRevokeCommand(any(), any());
				}
			}
		}

		@SneakyThrows
		@Test
		void shouldReturnForbiddenOnNonRevokedCommand() {
			var request = doRequest(CommandStatus.FINISHED);

			request.andExpect(status().isForbidden());
		}

		@SneakyThrows
		private ResultActions doRequest(CommandStatus status) {
			return mockMvc.perform(
					patch(LoeschAnforderungCommandController.BASE_PATH + "/{commandId}", LoeschAnforderungTestFactory.ID, CommandTestFactory.ID)
							.contentType(MediaType.APPLICATION_JSON_VALUE)
							.content(CommandTestFactory.buildStatusPatchContent(status)));
		}

		@DisplayName("handle vorgang zum loeschen markieren revoke command")
		@Nested
		class TestHandleVorgangZumLoeschenMarkierenRevokeCommand {

			private final StatusPatch statusPatch = StatusPatch.builder().build();

			@Test
			void shouldCallService() {
				handVorgangZumLoeschenMarkierenCommand();

				verify(loeschAnforderungService).deleteLoeschAnforderung(LoeschAnforderungTestFactory.ID);
			}

			@Test
			void shouldCallController() {
				handVorgangZumLoeschenMarkierenCommand();

				verify(commandController).revoke(CommandTestFactory.ID, statusPatch);
			}

			@Test
			void shouldReturnOk() {
				when(commandController.revoke(any(), any())).thenReturn(ResponseEntity.ok().build());

				var result = handVorgangZumLoeschenMarkierenCommand();

				assertThat(result.getStatusCode()).isEqualTo(HttpStatus.OK);
			}

			private ResponseEntity<EntityModel<Command>> handVorgangZumLoeschenMarkierenCommand() {
				return controller.handleVorgangZumLoeschenMarkierenRevokeCommand(LoeschAnforderungTestFactory.ID, CommandTestFactory.create(),
						statusPatch);
			}
		}

		@DisplayName("handle loeschAnforderung zuruecknehmen revoke command")
		@Nested
		class TestHandleLoeschVorgangZuruecknehmenRevokeCommand {

			private final StatusPatch statusPatch = StatusPatch.builder().build();
			private final Command command = CommandTestFactory.create();

			@Test
			void shouldCreateLoeschAnforderung() {
				controller.handleLoeschAnforderungZuruecknehmenRevokeCommand(command, statusPatch);

				verify(loeschAnforderungService).reCreateLoeschAnforderung(command);
			}

			@Test
			void shouldGetChangeStatusCommandId() {
				controller.handleLoeschAnforderungZuruecknehmenRevokeCommand(command, statusPatch);

				verify(loeschAnforderungService).getChangeStatusCommandId(command);
			}

			@Test
			void shouldCallController() {
				when(loeschAnforderungService.getChangeStatusCommandId(any())).thenReturn(CommandTestFactory.ID);

				controller.handleLoeschAnforderungZuruecknehmenRevokeCommand(command, statusPatch);

				verify(commandController).revoke(CommandTestFactory.ID, statusPatch);
			}

			@Test
			void shouldReturnOk() {
				when(commandController.revoke(any(), any())).thenReturn(ResponseEntity.ok().build());

				var result = controller.handleLoeschAnforderungZuruecknehmenRevokeCommand(command, statusPatch);

				assertThat(result.getStatusCode()).isEqualTo(HttpStatus.OK);
			}
		}
	}
}